%% Return which mouse buttons were pressed and keys were hit on the current
%% gnuplot graph.
%%
%% Syntax:
%%	[x, y, buttons] = ginput;
%%	[x, y, buttons] = ginput(n);
%% where the buttons field is optional.
%% If n is given, then wait for n presses or hits.
%% If n is not given, then loop until pressing the Return key; this
%% key does not go to the output.
%%
%% This is ginput.m implementation for gnuplot and X11.
%% It requires gnuplot 4.1 and later.
%%
%% Petr Mikulik
%% History: June 2006; August 2005; June 2004; April 2004
%% License: public domain

function [x, y, button] = ginput ( n )

if nargin > 1
    n = 100; % a startup-value
end

if nargout == 3
    button = 0;
end

if nargin > 1
    x = zeros(n, 1);
    y = zeros(n, 1);
    button = zeros(n, 1);
end

% gpinName = sprintf('./OctGnuplot%i', getpid);
gpinName = tmpnam();
[err, msg] = mkfifo(gpinName, 6*8*8); % 6*8*8) % 0600

k=0;
while 1
    k=k+1;
    graw(['set print "', gpinName, '";\n']);
    gpin = fopen(gpinName,'r');

    %graw('pause mouse "enjoy a mouse click"\n');
    graw('pause mouse any\n\n');

    % Previously: graw('print MOUSE_X, MOUSE_Y, MOUSE_KEY\n');
    % Notes: MOUSE_* can be undefined if user closes gnuplot by "q" or Alt-F4.
    % Further, this abrupt close also requires the leading "\n" on the next line.
    graw('\nif (exists("MOUSE_KEY") && exists("MOUSE_X")) print MOUSE_X, MOUSE_Y, MOUSE_KEY; else print "0 0 -1"\n');
    graw('set print\n'); % close output file, otherwise all blocks (why?!)

    % Now read from fifo
    %a = fgets(gpin,999) % for testing -- this shows the whole line
    [x(k), y(k), button(k), count] = fscanf(gpin, '%f %f %d', 'C');
    fclose(gpin);

    if 0
	printf('ginput.m: You pressed or hit %d at x=%f y=%f\n', button, x, y);
    end
    if ([x(k), y(k), button(k)] == [0, 0, -1])
	break; % Mousing not active (no plot until now)
    end

    if nargin>0 % input argument n was given => stop when k==n
        if (k==n) break; end
    else	% input argument n not given => stop when hitting a return key
        % if (button(k)==0x0D || button(k)==0x0A) % hit Return or Enter
        if (button(k)==0x0D) % hit Return
	    x(k:end)=[];
	    y(k:end)=[];
	    button(k:end)=[];
	    break;
	end
    end
end

unlink(gpinName);

% endfunction

% eof ginput.m
